-- SlashCommand by Altariel

-- TODO
-- * Add multiple command parsing (split commands on semicolon).
local VERSION = "0.1";

function SlashCommand_OnLoad()
	SLASH_SLASHADD1 = "/slashadd";
	SlashCmdList["SLASHADD"] = SlashCommand_SlashAddCmd;

	SLASH_SLASHDEL1 = "/slashdel";
	SLASH_SLASHDEL2 = "/slashrm";
	SlashCmdList["SLASHDEL"] = SlashCommand_SlashDelCmd;

	SLASH_SLASHREPLACE1 = "/slashreplace";
	SlashCmdList["SLASHREPLACE"] = SlashCommand_SlashReplaceCmd;

	if(string.find(VERSION,"BETA$") or string.find(VERSION,"UNSTABLE$") or tonumber(VERSION) < 1.0) then
		SLASH_SLASHPRINT1 = "/slashprint";
		SlashCmdList["SLASHPRINT"] = SlashCommand_PrintCommands;
	end

	this:RegisterEvent("VARIABLES_LOADED");

	if ( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand " .. VERSION .. " loaded.");
	end
end

function SlashCommand_OnEvent()
	if(event == "VARIABLES_LOADED") then
		if(not SlashCommandCommands) then
			SlashCommandCommands = {};
		end

		if(getn(SlashCommandCommands) > 0) then
			for i = 1, getn(SlashCommandCommands) do
				SlashCommand_LoadSlash(SlashCommandCommands[i].title);
			end
		end
	end
end	

function SlashCommand_SlashAddCmd(args)
	local a,b = string.find(args,"^%w+%s");
	if(a and b) then
		local cmd = string.sub(args,a,b-1);
		local text = string.sub(args,b+1,string.len(args));

		local ret = SlashCommand_AddSlash(cmd, text);

		if(ret) then
			DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Added command successfully");
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Usage: /slashadd name text");
	end

	for i = 1, getn(SlashCommandCommands) do
		SlashCommand_LoadSlash(SlashCommandCommands[i].title);
	end
end

function SlashCommand_SlashDelCmd(args)
	SlashCommand_DelSlash(args);
end

function SlashCommand_SlashReplaceCmd(args)
	local a,b = string.find(args,"^%w+%s");
	if(a and b) then
		local cmd = string.sub(args,a,b-1);
		local text = string.sub(args,b+1,string.len(args));

		local ret = SlashCommand_ReplaceSlash(cmd, text);

		if(ret) then
			DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Replaced command successfully");
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Usage: /slashreplace name text");
	end
end

function SlashCommand_AddSlash(command, text)
	local ret, err = SlashCommand_CheckDuplicates(command);
	if(ret) then
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand Error: " .. err);
		return false;
	end

	local num = getn(SlashCommandCommands);

	local tmp = {
		["title"] = command,
		["text"]  = text,
	}
	table.insert(SlashCommandCommands,tmp);
	SlashCommand_LoadSlash(command);

	if(getn(SlashCommandCommands) == num+1) then
		return true;
	else
		return false;
	end
end

function SlashCommand_LoadSlash(command)
	setglobal("SLASH_SC" .. string.upper(command) .. "1", "/" .. command);
	SlashCmdList["SC" .. string.upper(command)] = SlashCommand_ExeCommand;
end

function SlashCommand_UnloadSlash(command)
	setglobal("SLASH_SC" .. string.upper(command) .. "1", nil);
	SlashCmdList["SC" .. string.upper(command)] = nil;
end

function SlashCommand_DelSlash(command)
	for i = 1, getn(SlashCommandCommands) do
		if(SlashCommandCommands[i].title == command) then
			table.remove(SlashCommandCommands, i);
			SlashCommand_UnloadSlash(command);

			DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Removed command successfully");
			return;
		end
	end
end

function SlashCommand_Clear()
	SlashCommandCommands = {};
end

function SlashCommand_ReplaceSlash(command, text)
	local num = getn(SlashCommandCommands);

	for i = 1, num do
		if(SlashCommandCommands[i].title == command) then
			SlashCommandCommands[i] = {
				["title"] = command,
				["text"]  = text,
			}
		end
	end
	SlashCommand_LoadSlash(command);

	return true;
end

function SlashCommand_ExeCommand(args)
	local cmd = this:GetText();

	cmd = string.sub(cmd,2,string.len(cmd));

	for i = 1, getn(SlashCommandCommands) do
		if( SlashCommandCommands[i].title == cmd) then
			this:SetText(SlashCommandCommands[i].text);
			ChatEdit_OnEnterPressed();
		end
	end
end

function SlashCommand_CheckDuplicates(command)
	local cmds = getn(SlashCommandCommands);

	if(getn(SlashCommandCommands) > 0) then
		for i = 1, getn(SlashCommandCommands) do
			if(SlashCmdList["SC" .. string.upper(command)]) then
				return true,"Command '" .. command .. "' already exists";
			end
		end
	end
	return false;
end

function SlashCommand_PrintCommands()
	DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Current commands (total " .. getn(SlashCommandCommands) .. "):");
	for i = 1, getn(SlashCommandCommands) do
		DEFAULT_CHAT_FRAME:AddMessage(tostring(i) .. ": " .. SlashCommandCommands[i].title .. " => " .. SlashCommandCommands[i].text);
	end
end
